﻿'
' Copyright (c) 2021, MindFusion LLC - Bulgaria.
'

Imports System.Collections.Specialized
Imports System.Xml
Imports System.Windows.Media.Animation


''' <summary>
''' Interaction logic for MainWindow.xaml
''' </summary>
Partial Public Class MainWindow
    Inherits Window
    Public Sub New()
        InitializeComponent()

        siteMap.Bounds = New Rect(0, 0, 2000, 2000)
        siteMap.DefaultShape = Shapes.Rectangle

        startNewSite()
    End Sub

    Private Sub miNew_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        siteMap.ClearAll()
        startNewSite()
    End Sub

    Private Sub miSave_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Dim saveDlg As New Microsoft.Win32.SaveFileDialog()
        If saveDlg.ShowDialog() = True Then
            siteMap.SaveToXml(saveDlg.FileName)
        End If
    End Sub

    Private Sub siteMap_SerializeTag(ByVal sender As Object, ByVal e As SerializeTagEventArgs)
        Dim tag As Object = e.Tag
        If TypeOf tag Is PageProps Then
            Dim props As PageProps = DirectCast(e.Tag, PageProps)

            e.Context.WriteString(props.title, "Title", e.Representation)
            e.Context.WriteString(props.file, "File", e.Representation)
            e.Context.WriteString(props.img1Path, "Img1Path", e.Representation)
            e.Context.WriteString(props.img2Path, "Img2Path", e.Representation)
            e.Context.WriteString(props.inf.ToString(), "Inf", e.Representation)

            e.Handled = True
        End If
    End Sub

    Private Sub miLoad_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Try
            Dim openDlg As New Microsoft.Win32.OpenFileDialog()
            If openDlg.ShowDialog() = True Then
                Try
                    siteMap.LoadFromXml(openDlg.FileName)
                Catch
                    MessageBox.Show("Invalid file format.")
                    Return
                End Try
            End If

            ' find the home page
            For Each node As DiagramNode In siteMap.Nodes
                Dim props As PageProps = TryCast(node.Tag, PageProps)
                If props Is Nothing Then
                    props = New PageProps()
                    node.Tag = props
                End If
                If props.inf = -1 AndAlso TypeOf node Is ShapeNode Then
                    rootNode = TryCast(node, ShapeNode)
                    ' this is it
                    Exit For
                End If
            Next
        Catch generatedExceptionName As System.IO.FileLoadException
            MessageBox.Show(Me, "Invalid file format")
        Catch generatedExceptionName As XmlException
            MessageBox.Show(Me, "Invalid file format")
        End Try
    End Sub

    Private Sub siteMap_DeserializeTag(ByVal sender As Object, ByVal e As SerializeTagEventArgs)
        Dim tag As XmlElement = e.Representation
        If "Tag".Equals(tag.Name) AndAlso tag.ChildNodes.Count > 0 Then
            Dim props As New PageProps()

            props.title = e.Context.ReadString("Title", tag)
            props.file = e.Context.ReadString("File", tag)
            props.img1Path = e.Context.ReadString("Img1Path", tag)
            props.img2Path = e.Context.ReadString("Img2Path", tag)
            props.inf = e.Context.ReadInt("Inf", tag)

            e.Tag = props
            e.Handled = True
        End If
    End Sub

    Private Sub miGenerate_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        If rootNode.OutgoingLinks.Count = 0 Then
            MessageBox.Show("There are not links from the start page!")
            Return
        End If

        files.Clear()
        generateHtml(rootNode)

        ' display the generated html files
        For Each file As String In files.Keys
            Dim form As New PageViewForm()
            form.txHtml.Text = files(file)
            form.Title = file
            form.Show()
        Next
    End Sub

    Private Sub miExit_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Me.Close()
    End Sub

    Private Sub startNewSite()
        ' create the home page properties
        Dim props As New PageProps()
        props.title = "Home"
        props.file = "index.html"
        props.inf = -1

        ' create a node and associate it with the home page
        rootNode = New ShapeNode(siteMap)
        rootNode.Bounds = New Rect(40, 10, 80, 55)
        rootNode.Text = "Home"
        rootNode.Tag = props
        siteMap.Nodes.Add(rootNode)

        ' deselect the node; it gets selected because AutoSelect is turned on
        siteMap.Selection.Clear()

        fileNum = 1
    End Sub

    ' display 'property page' for a node
    Private Sub setPageProps(ByVal node As ShapeNode)
        Dim form As New PagePropsForm()
        form.WindowStartupLocation = WindowStartupLocation.CenterOwner

        ' get the page attributes
        Dim props As PageProps = DirectCast(node.Tag, PageProps)

        ' fill the form fields
        form.txTitle.Text = props.title
        form.txFileName.Text = props.file
        form.txImage1.Text = props.img1Path
        form.txImage2.Text = props.img2Path

        form.Owner = Me
        form.ShowDialog()

        ' get info from form fields
        node.Text = form.txTitle.Text
        props.title = form.txTitle.Text
        props.file = form.txFileName.Text
        props.img1Path = form.txImage1.Text
        props.img2Path = form.txImage2.Text

        node.InvalidateVisual()
    End Sub

    ' generate HTML code for a page
    Private Sub generateHtml(ByVal root As ShapeNode)
        Dim props As PageProps = DirectCast(root.Tag, PageProps)

        ' if not generated yet ...
        If files(props.file) Is Nothing Then
            Dim html As String = ""
            html += "<HTML>" & vbCr & vbLf
            html += "<HEAD>" & vbCr & vbLf
            html += "<TITLE>" & props.title & "</TITLE>" & vbCr & vbLf
            html += "</HEAD>" & vbCr & vbLf & vbCr & vbLf

            html += "<BODY>" & vbCr & vbLf & vbCr & vbLf

            ' add link to the parent page
            If root.IncomingLinks.Count > 0 Then
                Dim parentProps As PageProps = DirectCast(root.IncomingLinks(0).Origin.Tag, PageProps)
                html += "<A HREF=""" & parentProps.file & """>Level up: "
                html += parentProps.title & "</A><BR>" & vbCr & vbLf
            End If

            ' add link to all child pages
            For Each link As DiagramLink In root.OutgoingLinks
                Dim linkProps As PageProps = DirectCast(link.Destination.Tag, PageProps)
                html += "<A HREF=""" & linkProps.file & """>Level down: "
                html += linkProps.title & "</A><BR>" & vbCr & vbLf
            Next

            html += vbCr & vbLf & "</BODY>" & vbCr & vbLf
            html += "</HTML>" & vbCr & vbLf

            ' save the generated HTML
            files.Add(props.file, html)

            ' call recursively for all children
            For Each link As DiagramLink In root.OutgoingLinks
                generateHtml(DirectCast(link.Destination, ShapeNode))
            Next
        End If
    End Sub

    Private Sub siteMap_LinkCreated(ByVal sender As Object, ByVal e As MindFusion.Diagramming.Wpf.LinkEventArgs)
        If TypeOf e.Link.Destination Is DummyNode Then
            Dim pt As Point = e.Link.EndPoint
            Dim newNode As New ShapeNode(siteMap)
            newNode.Bounds = New Rect(pt.X - 40, pt.Y - 28, 80, 55)
            newNode.Opacity = 0
            siteMap.Nodes.Add(newNode)

            e.Link.Destination = newNode

            Me.newNode = newNode
            Dim animation As New DoubleAnimation(1, New Duration(TimeSpan.FromSeconds(0.4)))
            AddHandler animation.Completed, AddressOf OnAnimationCompleted
            newNode.BeginAnimation(ShapeNode.OpacityProperty, animation)

        End If
    End Sub

    Dim newNode As DiagramNode

    Sub OnAnimationCompleted(s1 As Object, e1 As EventArgs)
        OnNodeCreated(newNode)
    End Sub

    Private Sub siteMap_NodeClicked(ByVal sender As Object, ByVal e As MindFusion.Diagramming.Wpf.NodeEventArgs)
        ' we handle only right-clicks
        If e.MouseButton <> MouseButton.Right Then
            Return
        End If

        If Not (TypeOf e.Node Is ShapeNode) Then
            Return
        End If

        ' save the node reference
        currentNode = TryCast(e.Node, ShapeNode)

        ' display context menu for the node
        Dim cmNode As ContextMenu = createNodeMenu()
        cmNode.Placement = System.Windows.Controls.Primitives.PlacementMode.Mouse
        cmNode.IsOpen = True
    End Sub

    Private Function createNodeMenu() As ContextMenu
        Dim cmNode As New ContextMenu()

        Dim miProperties As New MenuItem()
        miProperties.Header = "Properties"
        AddHandler miProperties.Click, AddressOf miProperties_Click
        cmNode.Items.Add(miProperties)

        Dim miDelete As New MenuItem()
        miDelete.Header = "Delete"
        AddHandler miDelete.Click, AddressOf miDelete_Click
        cmNode.Items.Add(miDelete)

        Dim miView As New MenuItem()
        miView.Header = "View"
        AddHandler miView.Click, AddressOf miView_Click
        cmNode.Items.Add(miView)

        Return cmNode
    End Function

    Private Sub miView_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Dim props As PageProps = DirectCast(currentNode.Tag, PageProps)
        Process.Start("iexplore.exe", props.file)
    End Sub

    Private Sub miDelete_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Dim props As PageProps = DirectCast(currentNode.Tag, PageProps)
        If props.inf = -1 Then
            MessageBox.Show(Me, "Home page cannot be deleted!")
        Else
            siteMap.Nodes.Remove(currentNode)
        End If
    End Sub

    Private Sub miProperties_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        setPageProps(currentNode)
    End Sub

    Private Sub siteMap_NodeCreated(ByVal sender As Object, ByVal e As MindFusion.Diagramming.Wpf.NodeEventArgs)
        OnNodeCreated(TryCast(e.Node, ShapeNode))
    End Sub

    Private Sub siteMap_NodeDeleting(ByVal sender As Object, ByVal e As MindFusion.Diagramming.Wpf.NodeValidationEventArgs)
        ' get the associated page
        Dim props As PageProps = DirectCast(e.Node.Tag, PageProps)

        ' if that is the homepage ...
        If props.inf = -1 Then
            ' ... do not delete it
            e.Cancel = True
            MessageBox.Show(Me, "Home page cannot be deleted!")
        End If
    End Sub

    Private Sub OnNodeCreated(ByVal node As ShapeNode)
        ' create page object for the new node
        Dim props As New PageProps()
        props.file = "file" & fileNum.ToString() & ".html"
        props.title = "Page " & fileNum.ToString()
        node.Tag = props

        ' display the properties
        setPageProps(node)

        fileNum += 1
    End Sub

    Private rootNode As ShapeNode = Nothing
    Private currentNode As ShapeNode = Nothing

    ' generated HTML files are stored here
    Private files As New StringDictionary()

    ' page counter
    Private fileNum As Integer = 1
End Class

' objects of this type will be associated with nodes as tags.
' as the class is marked as serializable, its objects will be saved
' together with the associated nodes
<Serializable()> _
Friend Class PageProps
    Friend title As String = "New page"
    Friend file As String = ""
    Friend img1Path As String = ""
    Friend img2Path As String = ""
    Friend inf As Integer = 0
End Class